@@ -5,20 +5,21 @@ import shutil
 from base64 import b64decode
 from contextlib import suppress
 from datetime import datetime
-from functools import partial
+from functools import partial, cached_property
 from http.cookiejar import Cookie
 from http.cookies import SimpleCookie
 from io import BytesIO
 from json import JSONDecoder, loads
 
 from dateutil.parser import parse
-from requests import Request, Response
+from requests import Request, Response, HTTPError
 from typing import Dict, Optional, Iterable, Union, TextIO
 
 from requests.structures import CaseInsensitiveDict
 
 from .scrape_config import ScrapeConfig
-from .errors import ErrorFactory, UpstreamHttpClientError, EncoderError
+from .errors import ErrorFactory, EncoderError, ApiHttpClientError, ApiHttpServerError, \
+    HttpError
 from .frozen_dict import FrozenDict
 
 logger.getLogger('scrapfly')
@@ -96,11 +97,38 @@ class ResponseBodyHandler:
 
 class ScrapeApiResponse:
 
-    def __init__(self, request: Request, response: Response, scrape_config: ScrapeConfig,
-                 api_result: Optional[Dict] = None):
+    def __init__(self, request: Request, response: Response, scrape_config: ScrapeConfig, api_result: Optional[Dict] = None):
         self.request = request
         self.response = response
         self.scrape_config = scrape_config
+
+        if self.scrape_config.method == 'HEAD':
+            api_result = {
+                'result': {
+                    'request_headers': {},
+                    'success': 200 >= self.response.status_code < 300,
+                    'response_headers': self.response.headers,
+                    'status_code': self.response.status_code,
+                    'reason': self.response.reason,
+                    'format': 'text',
+                    'content': ''
+                },
+                'context': {},
+                'config': self.scrape_config.__dict__
+            }
+
+            if 'X-Scrapfly-Reject-Code' in response.headers:
+                api_result['result']['error'] = {
+                    'code': response.headers['X-Scrapfly-Reject-Code'],
+                    'http_code': response.headers['X-Scrapfly-Reject-Http-Code'],
+                    'message': response.headers['X-Scrapfly-Reject-Description'],
+                    'error_id': response.headers['X-Scrapfly-Reject-ID'],
+                    'retryable': True if response.headers['X-Scrapfly-Reject-Retryable'] == 'yes' else False
+                }
+
+                if 'X-Scrapfly-Reject-Doc' in response.headers:
+                    api_result['result']['error']['doc_url'] = response.headers['X-Scrapfly-Reject-Doc']
+
         self.result = self.handle_api_result(api_result=api_result)
 
     @property
@@ -119,10 +147,21 @@ class ScrapeApiResponse:
     def content(self) -> str:
         return self.scrape_result['content']
 
-    # /!\ Success means scrapfly api reply correctly to the call, but the scrape can be unsuccessful if the upstream reply with error status code
     @property
     def success(self) -> bool:
-        return self.is_scrape_engine_error is False and self.is_api_error is False
+        """
+            /!\ Success means Scrapfly api reply correctly to the call, but the scrape can be unsuccessful if the upstream reply with error status code
+        """
+        return 200 >= self.response.status_code <= 299
+
+    @property
+    def scrape_success(self) -> bool:
+        return self.scrape_result['success']
+
+    @property
+    def error(self) -> Optional[Dict]:
+        if self.scrape_success is False:
+            return self.scrape_result['error']
 
     @property
     def status_code(self) -> int:
@@ -132,24 +171,7 @@ class ScrapeApiResponse:
     def headers(self) -> CaseInsensitiveDict:
         return self.response.headers
 
-    def handle_api_result(self, api_result: Optional[Dict]) -> Optional[FrozenDict]:
-        if not api_result and self.scrape_config.method != 'HEAD':
-            return None
-
-        if self.scrape_config.method == 'HEAD':
-            api_result = {
-                'result': {
-                    'request_headers': {},
-                    'response_headers': self.response.headers,
-                    'status_code': self.response.status_code,
-                    'reason': self.response.reason,
-                    'format': 'text',
-                    'content': ''
-                },
-                'context': {},
-                'config': self.scrape_config.__dict__
-            }
-
+    def handle_api_result(self, api_result: Dict) -> Optional[FrozenDict]:
         if self._is_api_error(api_result=api_result) is True:
             return FrozenDict(api_result)
 
@@ -165,21 +187,14 @@ class ScrapeApiResponse:
 
         return FrozenDict(api_result)
 
-    def _is_scrape_engine_error(self, api_result: Optional[Dict] = None) -> bool:
-        if api_result is None:
-            return False
-
-        if 'result' not in api_result:
-            return False
-
+    @cached_property
+    def selector(self):
         try:
-            return api_result['result']['error'] is not None
-        except KeyError:
-            return False
-
-    @property
-    def is_scrape_engine_error(self) -> bool:
-        return self._is_scrape_engine_error(self.result)
+            from scrapy import Selector
+            return Selector(text=self.content)
+        except ImportError as e:
+            logger.error('You must install scrapfly[scrapy] to enable this feature')
+            raise e
 
     def _is_api_error(self, api_result: Dict) -> bool:
         if self.scrape_config.method == 'HEAD':
@@ -192,45 +207,63 @@ class ScrapeApiResponse:
 
         return 'error_id' in api_result
 
-    @property
-    def is_api_error(self) -> bool:
-        return self._is_api_error(self.result)
-
     def raise_for_result(self, raise_on_upstream_error: bool = True):
-        if self.is_api_error is True:
-            raise ErrorFactory.create(api_response=self)
 
-        if self.is_scrape_engine_error is True:
+        try:
+            self.response.raise_for_status()
+        except HTTPError as e:
+            if 'http_code' in self.result:
+                if e.response.status_code >= 500:
+                    raise ApiHttpServerError(
+                        request=e.request,
+                        response=e.response,
+                        message= self.result['message'],
+                        code='',
+                        resource='',
+                        http_status_code=e.response.status_code
+                    ) from e
+                else:
+                    raise ApiHttpClientError(
+                        request=e.request,
+                        response=e.response,
+                        message=self.result['message'],
+                        code='',
+                        resource='API',
+                        http_status_code=self.result['http_code']
+                    ) from e
+
+        if self.scrape_success is False:
             error = ErrorFactory.create(api_response=self)
 
-            if isinstance(error, UpstreamHttpClientError) is True:
-                if raise_on_upstream_error is True:
+            if error:
+                if isinstance(error, HttpError):
+                    if raise_on_upstream_error is True:
+                        raise error
+                else:
                     raise error
-            else:
-                raise error
 
     def upstream_result_into_response(self, _class=Response) -> Optional[Response]:
         if _class != Response:
             raise RuntimeError('only Response from requests package is supported at the moment')
+
         if self.result is None:
             return None
 
-        if self.is_scrape_engine_error is True:  # Upstream website must have reply
+        if self.response.status_code != 200:
             return None
 
         response = Response()
-        response.status_code = self.result['result']['status_code']
-        response.reason = self.result['result']['reason']
-        response._content = self.result['result']['content'].encode('utf-8') if self.result['result'][
-            'content'] else None
-        response.headers.update(self.result['result']['response_headers'])
-        response.url = self.result['result']['url']
+        response.status_code = self.scrape_result['status_code']
+        response.reason = self.scrape_result['reason']
+        response._content = self.scrape_result['content'].encode('utf-8') if self.scrape_result['content'] else None
+        response.headers.update(self.scrape_result['response_headers'])
+        response.url = self.scrape_result['url']
 
         response.request = Request(
-            method=self.result['config']['method'],
-            url=self.result['config']['url'],
-            headers=self.result['result']['request_headers'],
-            data=self.result['config']['body'] if self.result['config']['body'] else None
+            method=self.config['method'],
+            url=self.config['url'],
+            headers=self.scrape_result['request_headers'],
+            data=self.config['body'] if self.config['body'] else None
         )
 
         if 'set-cookie' in response.headers:
@@ -265,8 +298,7 @@ class ScrapeApiResponse:
                         port=None,
                         port_specified=False,
                         domain_specified=cookie.get('domain') is not None and cookie.get('domain') != '',
-                        domain_initial_dot=bool(cookie.get('domain').startswith('.')) if cookie.get(
-                            'domain') is not None else False,
+                        domain_initial_dot=bool(cookie.get('domain').startswith('.')) if cookie.get('domain') is not None else False,
                         path_specified=cookie.get('path') != '' and cookie.get('path') is not None,
                         discard=False,
                         comment_url=None,
@@ -279,8 +311,7 @@ class ScrapeApiResponse:
 
         return response
 
-    def sink(self, path: Optional[str] = None, name: Optional[str] = None,
-             file: Optional[Union[TextIO, BytesIO]] = None):
+    def sink(self, path: Optional[str] = None, name: Optional[str] = None, file: Optional[Union[TextIO, BytesIO]] = None):
         file_content = self.scrape_result['content']
         file_path = None
         file_extension = None
