@@ -6,7 +6,7 @@ import http
 import platform
 import re
 import shutil
-from functools import partial
+from functools import partial, cached_property
 from io import BytesIO
 
 import backoff
@@ -17,13 +17,13 @@ import requests
 import urllib3
 import logging as logger
 
-logger.getLogger('scrapfly')
-
 from .errors import *
 from .api_response import ResponseBodyHandler
 from .scrape_config import ScrapeConfig
 from . import __version__ as version, ScrapeApiResponse
 
+logger.getLogger('scrapfly')
+
 NetworkError = (
     ConnectionError,
     RequestExceptions.ConnectionError,
@@ -84,6 +84,10 @@ class ScrapflyClient:
         if self.debug is True:
             http.client.HTTPConnection.debuglevel = 5
 
+    @cached_property
+    def _http_handler(self):
+        return partial(self.http_session.request if self.http_session else requests.request)
+
     def _scrape_request(self, scrape_config:ScrapeConfig):
 
         if self.distributed_mode is True and scrape_config.correlation_id is None:
@@ -105,8 +109,7 @@ class ScrapflyClient:
         }
 
     def account(self) -> Union[str, Dict]:
-        http_handler = partial(self.http_session.request if self.http_session else requests.request)
-        response = http_handler('GET', self.host + '/account', params={'key': self.key})
+        response = self._http_handler('GET', self.host + '/account', params={'key': self.key})
 
         response.raise_for_status()
 
@@ -140,6 +143,8 @@ class ScrapflyClient:
     def open(self):
         if self.http_session is None:
             self.http_session = Session()
+            self.http_session.verify = self.verify
+            self.http_session.timeout = (self.connect_timeout, self.read_timeout)
             self.http_session.params['key'] = self.key
             self.http_session.headers['accept-encoding'] = self.body_handler.content_encoding
             self.http_session.headers['accept'] = self.body_handler.accept
@@ -189,8 +194,7 @@ class ScrapflyClient:
     def scrape(self, scrape_config:ScrapeConfig) -> ScrapeApiResponse:
         logger.debug('--> %s Scrapping %s' % (scrape_config.method, scrape_config.url))
         request_data = self._scrape_request(scrape_config=scrape_config)
-        http_handler = partial(self.http_session.request if self.http_session else requests.request)
-        response = http_handler(**request_data)
+        response = self._http_handler(**request_data)
         return self._handle_response(response=response, scrape_config=scrape_config)
 
     def _handle_response(self, response:Response, scrape_config:ScrapeConfig) -> ScrapeApiResponse:
@@ -213,24 +217,20 @@ class ScrapflyClient:
                 )
 
             return api_response
-        except ApiHttpServerError as e:
-            logger.critical('<-- %s - %s' % (e.response.status_code, str(e)))
-            raise
-        except UpstreamHttpServerError as e:
+        except UpstreamHttpClientError as e:
             if scrape_config.method == 'HEAD':
-                logger.warning('<-- %s - %s | %s' % (e.code, str(e), e.api_response.response.request.url))
+                logger.warning('<-- %s | %s' % (str(e), e.api_response.response.request.url))
             else:
-                logger.warning('<-- %s - %s | %s' % (e.code, str(e), e.api_response.result['result']['url']))
-
-                if scrape_config.asp is False and e.api_response.status_code in [403, 429, 999]:
-                    logger.warning('Enable ASP to bypass anti bot solution for better success rate. Checkout: https://scrapfly.io/docs/scrape-api/anti-scraping-protection')
-
+                logger.warning('<-- %s | %s' % (str(e), e.api_response.result['result']['url']))
+            raise
+        except ApiHttpServerError as e:
+            logger.critical('<-- %s' % str(e))
             raise
         except ScrapflyError as e:
-            logger.critical('<-- %s - %s' % (e.code, str(e)))
+            logger.critical('<-- - %s' % (str(e)))
             raise
 
-    def screenshot(self, url:str, path:Optional[str]=None, name:Optional[str]=None):
+    def screenshot(self, url:str, path:Optional[str]=None, name:Optional[str]=None) -> str:
         # for advance configuration, take screenshots via scrape method with ScrapeConfig
         api_response = self.scrape(scrape_config=ScrapeConfig(
             url=url,
@@ -253,9 +253,9 @@ class ScrapflyClient:
 
             screenshot = response.content
 
-        self.sink(api_response, path=path, name=name, content=screenshot)
+        return self.sink(api_response, path=path, name=name, content=screenshot)
 
-    def sink(self, api_response:ScrapeApiResponse, content:Optional[Union[str, bytes]]=None, path: Optional[str] = None, name: Optional[str] = None, file: Optional[Union[TextIO, BytesIO]] = None):
+    def sink(self, api_response:ScrapeApiResponse, content:Optional[Union[str, bytes]]=None, path: Optional[str] = None, name: Optional[str] = None, file: Optional[Union[TextIO, BytesIO]] = None) -> str:
         scrape_result = api_response.result['result']
         scrape_config = api_response.result['config']
 
@@ -310,6 +310,7 @@ class ScrapflyClient:
             shutil.copyfileobj(file_content, f, length=131072)
 
         logger.info('file %s created' % file_path)
+        return file_path
 
     def _handle_api_response(
         self,
@@ -318,15 +319,18 @@ class ScrapflyClient:
         raise_on_upstream_error: Optional[bool] = True
     ) -> ScrapeApiResponse:
 
-        if self.body_handler.support(headers=response.headers):
-            result = self.body_handler(response.content)
+        if scrape_config.method == 'HEAD':
+            body = None
         else:
-            result = response.content.decode('utf-8')
+            if self.body_handler.support(headers=response.headers):
+                body = self.body_handler(response.content)
+            else:
+                body = response.content.decode('utf-8')
 
         api_response:ScrapeApiResponse = ScrapeApiResponse(
             response=response,
             request=response.request,
-            api_result=result,
+            api_result=body,
             scrape_config=scrape_config
         )
 
